'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _gm = require('gm');

var _gm2 = _interopRequireDefault(_gm);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _rimraf = require('rimraf');

var _rimraf2 = _interopRequireDefault(_rimraf);

var _generateUUID = require('./helpers/generateUUID');

var _generateUUID2 = _interopRequireDefault(_generateUUID);

var _scroll = require('./scripts/scroll');

var _scroll2 = _interopRequireDefault(_scroll);

var _getPageInfo = require('./scripts/getPageInfo');

var _getPageInfo2 = _interopRequireDefault(_getPageInfo);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; } /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * Save a screenshot as a base64 encoded PNG with the current state of the browser.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * <example>
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             :saveScreenshot.js
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                             client
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 // set browser window size
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 .windowHandleSize({width: 500, height: 500})
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 .saveScreenshot('viewport.png') // make screenshot of current viewport (500x500px)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 .saveScreenshot('wholeScreen.png', true) // makes screenshot of whole document (1280x1342px)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                 .end();
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * </example>
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * @param {String}   fileName    path of generated image (relative to the execution directory)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * @param {Boolean=} totalScreen if true (default value) it takes a screenshot of whole website, otherwise only of current viewport
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * @uses protocol/execute, utility/scroll, protocol/screenshot
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          * @type utility
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                          */

var CAPS_TAKING_FULLSIZE_SHOTS = ['firefox'];

exports.default = function () {
    var _ref = _asyncToGenerator(function* (fileName) {
        var _this = this;

        var ErrorHandler = this.instance.ErrorHandler;

        /*!
         * parameter check
         */
        if (typeof fileName !== 'string') {
            throw new ErrorHandler.CommandError('filename from type string is require, got ' + fileName);
        }

        var cropImages = [];
        var currentXPos = 0;
        var currentYPos = 0;
        var screenshot = null;

        /*!
         * create tmp directory to cache viewport shots
         */
        var uuid = (0, _generateUUID2.default)();
        var tmpDir = _path2.default.join(__dirname, '..', '.tmp-' + uuid);
        // Todo: promisify this
        if (!_fs2.default.existsSync(tmpDir)) {
            _fs2.default.mkdirSync(tmpDir, '0755');
        }

        /*!
         * prepare page scan
         */
        var pageInfo = (yield this.instance.execute(_getPageInfo2.default)).value;

        /**
         * no need to stitch viewports together if screenshot is from the whole website
         */
        if (CAPS_TAKING_FULLSIZE_SHOTS.indexOf(this.browserName) === -1) {
            var _screenshot = yield this.instance.saveScreenshot();
            _fs2.default.writeFileSync(fileName, _screenshot);
            return _screenshot;
        }

        /*!
         * run scan
         */

        var _loop = function* _loop() {
            /*!
             * take screenshot of viewport
             */
            var shot = yield _this.screenshot();

            /*!
             * cache viewport image into tmp dir
             */
            var file = tmpDir + '/' + currentXPos + '-' + currentYPos + '.png';
            var gmImage = (0, _gm2.default)(new Buffer(shot.value, 'base64'));

            if (pageInfo.devicePixelRatio > 1) {
                percent = 100 / pageInfo.devicePixelRatio;

                gmImage.resize(percent, percent, '%');
            }

            gmImage.crop(pageInfo.screenWidth, pageInfo.screenHeight, 0, 0);

            if (!cropImages[currentXPos]) {
                cropImages[currentXPos] = [];
            }

            cropImages[currentXPos][currentYPos] = file;

            currentYPos++;
            if (currentYPos > Math.floor(pageInfo.documentHeight / pageInfo.screenHeight)) {
                currentYPos = 0;
                currentXPos++;
            }

            yield new Promise(function (resolve) {
                return gmImage.write(file, resolve);
            });

            /*!
             * scroll to next area
             */
            yield _this.instance.execute(_scroll2.default, currentXPos * pageInfo.screenWidth, currentYPos * pageInfo.screenHeight);

            /**
             * have a small break to allow browser to render
             */
            yield _this.instance.pause(50);
        };

        while (currentXPos < pageInfo.documentWidth / pageInfo.screenWidth) {
            var percent;
            yield* _loop();
        }

        /*!
         * concats all shots
         */
        var subImg = 0;

        for (var snippet in cropImages) {
            var col = (0, _gm2.default)(snippet.shift());
            col.append.apply(col, snippet);

            if (!screenshot) {
                screenshot = col;
                yield new Promise(function (resolve) {
                    return col.write(fileName, resolve);
                });
            } else {
                yield new Promise(function (resolve) {
                    return col.write(tmpDir + '/' + ++subImg + '.png', function () {
                        return (0, _gm2.default)(fileName).append(tmpDir + '/' + subImg + '.png', true).write(fileName, resolve);
                    });
                });
            }
        }

        /*!
         * crop screenshot regarding page size
         */
        yield new Promise(function (resolve) {
            return (0, _gm2.default)(fileName).crop(pageInfo.documentWidth, pageInfo.documentHeight, 0, 0).write(fileName, resolve);
        });

        /*!
         * remove tmp dir
         * Todo: promisify
         */
        _rimraf2.default.sync(tmpDir);

        /*!
         * scroll back to start position
         */
        yield this.execute(_scroll2.default, 0, 0);
    });

    function documentScreenshot(_x) {
        return _ref.apply(this, arguments);
    }

    return documentScreenshot;
}();
