'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

var fs = require('fs'),
    promisify = require('es6-promisify'),
    logWarning = require('./logWarning.js');

var rename = promisify(fs.rename);

module.exports = function () {
    var _ref = _asyncToGenerator(function* (imageDiff) {
        var _this = this;

        if (!imageDiff) {
            this.self.options.resultObject[this.currentArgs.name].push({
                baselinePath: this.baselinePath,
                message: 'first image of module "' + this.currentArgs.name + '" from page "' + this.pagename + '" successfully taken',
                misMatchPercentage: 0,
                isExactSameImage: true,
                isSameDimensions: true,
                isWithinMisMatchTolerance: true,
                properties: this.currentArgs
            });

            return;
        }

        var misMatchTolerance = parseFloat(imageDiff.misMatchPercentage, 10);

        /**
         * if set misMatchTolerance is smaller then compared misMatchTolerance
         * make image diff
         */
        if (this.misMatchTolerance < misMatchTolerance) {

            /*istanbul ignore next*/
            if (!imageDiff.isSameDimensions) {
                logWarning.call(this.instance, 'DimensionWarning');
            }

            this.self.options.resultObject[this.currentArgs.name].push({
                baselinePath: this.baselinePath,
                regressionPath: this.regressionPath,
                diffPath: this.diffPath,
                message: 'mismatch tolerance exceeded (+' + (misMatchTolerance - this.misMatchTolerance) + '), image-diff created',
                misMatchPercentage: misMatchTolerance,
                isExactSameImage: false,
                isSameDimensions: imageDiff.isSameDimensions,
                isWithinMisMatchTolerance: false,
                properties: this.currentArgs
            });

            yield new Promise(function (resolve) {
                imageDiff.getDiffImage().pack().on('end', function () {
                    return resolve(_this.resultObject);
                }).pipe(fs.createWriteStream(_this.diffPath));
            });
        } else {
            /**
             * otherwise delete diff
             */

            if (fs.existsSync(this.diffPath)) {
                fs.unlinkSync(this.diffPath);
            }

            /**
             * Save a new baseline image, if one doesn't already exist.
             *
             * If one does exist, we delete the temporary regression.
             */
            if (fs.existsSync(this.baselinePath)) {
                fs.unlinkSync(this.regressionPath);
            } else {
                rename(this.regressionPath, this.baselinePath);
            }

            /**
             * return result object to WebdriverIO instance
             */
            this.self.options.resultObject[this.currentArgs.name].push({
                baselinePath: this.baselinePath,
                message: 'mismatch tolerance not exceeded (~' + misMatchTolerance + '), baseline didn\'t change',
                misMatchPercentage: misMatchTolerance,
                isExactSameImage: misMatchTolerance === 0,
                isSameDimensions: imageDiff.isSameDimensions,
                isWithinMisMatchTolerance: true
            });
        }
    });

    return function (_x) {
        return _ref.apply(this, arguments);
    };
}();
