'use strict';

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { return step("next", value); }, function (err) { return step("throw", err); }); } } return step("next"); }); }; }

var endSession = require('./endSession.js');

/**
 * run regression test
 */
module.exports = function () {
    var _ref = _asyncToGenerator(function* (pagename, args) {
        this.needToSync = true;

        /*istanbul ignore next*/
        if (typeof args === 'function') {
            args = {};
        }

        if (!(args instanceof Array)) {
            args = [args];
        }

        /**
         * parameter type check
         */
        /*istanbul ignore next*/
        if (typeof pagename === 'function') {
            throw new Error('A pagename is required');
        }
        /*istanbul ignore next*/
        if (typeof args[0].name !== 'string') {
            throw new Error('You need to specify a name for your visual regression component');
        }

        var queuedShots = JSON.parse(JSON.stringify(args));

        while (queuedShots.length > 0) {
            var currentArgs = queuedShots[0];

            var context = {
                self: this,

                /**
                 * default attributes
                 */
                misMatchTolerance: this.options.misMatchTolerance,
                screenshotRoot: this.options.screenshotRoot,
                failedComparisonsRoot: this.options.failedComparisonsRoot,

                instance: this.instance,
                pagename: pagename,
                applitools: {
                    host: this.options.applitools.host,
                    apiKey: this.options.applitools.apiKey,
                    appName: pagename,
                    baselineName: this.options.applitools.baselineName,
                    saveNewTests: this.options.applitools.saveNewTests,
                    saveFailedTests: this.options.applitools.saveFailedTests,
                    batchId: this.options.applitools.batchId // Group all sessions for this instance together.
                },
                currentArgs: currentArgs,
                queuedShots: queuedShots,
                baselinePath: this.options.screenshotRoot + '/' + pagename + '.' + currentArgs.name + '.baseline.png',
                regressionPath: this.options.screenshotRoot + '/' + pagename + '.' + currentArgs.name + '.regression.png',
                diffPath: this.options.failedComparisonsRoot + '/' + pagename + '.' + currentArgs.name + '.diff.png',
                screenshot: this.options.screenshotRoot + '/' + pagename + '.png',
                isComparable: false,
                warnings: [],
                newScreenSize: 0,
                pageInfo: null,
                updateBaseline: typeof currentArgs.updateBaseline === 'boolean' ? currentArgs.updateBaseline : this.options.updateBaseline,
                screenWidth: currentArgs.screenWidth || [].concat(this.options.screenWidth) };

            /**
             * initiate result object
             */
            if (!this.options.resultObject[currentArgs.name]) {
                this.options.resultObject[currentArgs.name] = [];
            }

            // start applitools session
            yield require('./startSession.js').call(context);

            /**
             * if multiple screen width are given resize browser dimension
             */
            yield require('./setScreenWidth.js').call(context);

            /**
             * make screenshot via [GET] /session/:sessionId/screenshot
             */
            yield require('./makeScreenshot.js').call(context);

            /**
             * check if files with id already exists
             */
            yield require('./renameFiles.js').call(context);

            /**
             * get page informations
             */
            var pageInfo = yield require('./getPageInfo.js').call(context);

            /**
             * crop image according to user arguments and its position on screen and save it
             */
            yield require('./cropImage.js').call(context, pageInfo);

            /**
             * compare images
             */
            var imageDiff = yield require('./compareImages.js').call(context);

            yield require('./saveImageDiff.js').call(context, imageDiff);

            // move on to the next screenshot only when the number of screenshot
            // dimensions has been exhausted
            if (!context.screenWidth || !context.screenWidth.length) {
                yield endSession.call(context);
                queuedShots.shift();
            }
        }

        this.takeScreenshot = undefined;

        var result = this.options.resultObject;
        this.options.resultObject = {};
        return result;
    });

    return function (_x, _x2) {
        return _ref.apply(this, arguments);
    };
}();
